%% Identify battery capacity from measurements.
% A123 cell @ 25 degrees
%% a)

clc
close all
clear all


load("A123_low_current_data.mat");

% Intervals in the data that are relevant
interval_all = 2:33000;
interval_1 = 2:452;
interval_2 = 453:1592;
interval_3 = 1593:1652;
interval_4 = 1653:17318; % Discharging
interval_5 = 17319:17378;
interval_6 = 17379:33000; % Charging 

inte = 1:length(data(:,1));

plot_int = interval_4; % Can be changed to look at other intervalls

figure(10)
yyaxis left
plot(data(plot_int,1),data(plot_int,2))
xlabel('Time [s]')
ylabel('Current [A]')
yyaxis right 
plot(data(plot_int,1),data(plot_int,3))
ylabel('Voltage [V]')

Q_discharge = 0;

for i = 1653:17318-1
    Q_discharge = Q_discharge + (data(i+1,1)-data(i,1))/3600*(data(i+1,2)+data(i,2))/2;
end

Q_charge = 0;

for i = 17379:33000-1
    Q_charge = Q_charge + (data(i+1,1)-data(i,1))/3600*(data(i+1,2)+data(i,2))/2;
end

Q_bat = (Q_charge - Q_discharge)/2;

%%

discharge_ind = interval_4;
charge_ind = interval_6;
Time = data(:,1);
Current = data(:,2);
Voltage = data(:,3);

OCVf = @(soc_vec,v_vec,soc) interp1(soc_vec, v_vec, soc, 'spline','extrap');

% Estimate the capacity for the charge and discharge period
Q_discharge = cumtrapz(Time(discharge_ind)/3600, Current(discharge_ind));
Q_charge = cumtrapz(Time(charge_ind)/3600, Current(charge_ind));

% Estimate the SOC for the charge and discharge period
SOC_dis = (max(abs(Q_discharge)) - abs(Q_discharge))/max(abs(Q_discharge));
SOC_ch = abs(Q_charge)/max(abs(Q_charge));

% The voltage for the charge and discharge period
Vdis = Voltage(discharge_ind);
Vch = Voltage(charge_ind);

SOC_vec = 0:0.001:1; % Create a SOC vector

% Create the OCV curve for the different periods
OCV_charge = OCVf(SOC_ch, Vch, SOC_vec);
OCV_discharge = OCVf(SOC_dis, Vdis, SOC_vec);

OCV = (OCV_discharge + OCV_charge)/2;

figure;
plot(SOC_vec,OCV_charge)
hold on
plot(SOC_vec,OCV_discharge)
plot(SOC_vec,OCV)
xlabel('Time [s]')
ylabel('Terminal Voltage [V]')
legend('Charge', 'Discharge','True OCV')