close all
clear all
clc

do_plots = 0
%%
% Use determined OCV-SOC data from previous exerice
load('OCV_SOC_data.mat') % in phd_battery_systems\Lessons\Matlab

% Use the data for the functions (if not already defined)
OCVf = @(z) interp1(SOC_grd,OCVi,z);
SOCf = @(v) interp1(OCVi,SOC_grd,v);

%% Load data
load('A123_dynamic_data_sample_2.mat')

t = Time;
i = Current;
v = Voltage;

%% Extract the training data = dynamic data of the first discharge 

close all;
dynamic = find(StepIndex >= 8 & StepIndex <= 10);
dynamic = dynamic(1:floor(0.93*numel(dynamic)));
tdyn = t(dynamic);
vdyn = v(dynamic);
idyn = i(dynamic); 

%% Assign Ri, R and C a value

Ri = 0.1960;
C = 1016.9;
R = 0.0791;
Q = 1.0865*3600; % Capacity (1.0865 Ah) from another exercise

%% Select data set
% only dynamic part
idx = 1:5000;      % part of the discharge  
%idx = 1:numel(tmp); % hole discharge
idx = dynamic(idx); 
%idx = 1:numel(Voltage);% all data

training_data.v = Voltage(idx)';
training_data.i = Current(idx)';
training_data.t = Time(idx)';

%% Evaluate one RC models.
% =========================================================================
% Ri,R,C, Q, inital SOC from Linear regression. 
th = ones(5,1);
params0 = [Ri;C;R;SOCf(training_data.v(1));Q];
[r,vhat] = ECM1RC(th,training_data,OCVf,params0);
param_result1 = [params0;r]
%% Task a) Initial values from LinReg. Optimizing Ri,R,C. Unnormalized
params0 = [ones(3,1);SOCf(training_data.v(1));Q];
th = [Ri;C;R]; 
fun = @(x) ECM1RC(x,training_data,OCVf,params0);
[x,r] = fminunc(fun,th)
param_result1(:,2) = [x;params0(4:5);r]

%% Task b) Initial values from LinReg. Optimizing Ri,R,C. Normalized
params0 = [Ri;C;R;SOCf(training_data.v(1));Q];
th = ones(3,1);
fun = @(x) ECM1RC(x,training_data,OCVf,params0);
[x,r] = fminunc(fun,th)
param_result1(:,3) = [x.*params0(1:3);params0(4:5);r]

%% Task c) Initial values from LinReg. Optimizing Ri,R,C,inital SOC. Unnormalized
params0 = [ones(4,1);Q];
th = [Ri;C;R;SOCf(training_data.v(1))]; 
fun = @(x) ECM1RC(x,training_data,OCVf,params0);
[x,r] = fminunc(fun,th)
param_result1(:,4) = [x;params0(5);r]

%% Task d) Initial values from LinReg. Optimizing Ri,R,C,inital SOC. Normalized
params0 = [Ri;C;R;SOCf(training_data.v(1));Q];
th = ones(4,1); 
fun = @(x) ECM1RC(x,training_data,OCVf,params0);
[x,r] = fminunc(fun,th)
param_result1(:,5) = [x.*params0(1:4);params0(5);r]

%% Task e) Initial values from LinReg. Optimizing Ri,R,C,inital SOC, Q.Normalized 
params0 = [Ri;C;R;SOCf(training_data.v(1));Q];
th = ones(5,1);
fun = @(x) ECM1RC(x,training_data,OCVf,params0);
[x,r] = fminunc(fun,th)
param_result1(:,6) = [x.*params0(1:5);r]

%% Plot model


figure(1);
params = param_result1(:,end);
th = ones(6,1);
[~,vhat] = ECM1RC(th,training_data,OCVf,params);
plot(training_data.t,[vhat training_data.v])
xlabel('Time [s]')
ylabel('Voltage [V]')

figure(2);clf

plot(Time(idx),training_data.v-vhat)
xlabel('Time [s]')


%%
function [r,vhat] = ECM1RC(th,data,OCVf,params0)

% Init
v = data.v;
i = data.i;
dt = diff(data.t);

n = numel(i);    % Insignal i = [i_1, ..., i_n]'% ; 
vc = zeros(n,1); % Initialvillkor vc(1) = 0
z = zeros(n,1);
vhat = zeros(n,1); 

% Set parameters
x = ones(5,1);
x(1:numel(th)) = th;
params = x.*params0;
Ri = params(1);
C  = params(2);
R  = params(3);
z(1) = params(4);
Q = params(5);

% Simulate model
for k = 1:n-1
    z(k+1) = z(k) + dt(k) * i(k)/Q;
    vhat(k) =  vc(k) + OCVf(z(k)) + Ri*i(k);
    vc(k+1) = vc(k)*(1-dt(k)/(R*C)) + 1/C*i(k)*dt(k);
end
vhat(n) =  vc(n) + OCVf(z(n)) + Ri*i(n);

% Evaluate error
r = mean((v-vhat).^2); 
end