clc
close all
clear all


data = load("A123_dynamic_data_sample_2.mat");

% Use determined OCV-SOC data from previous exerice
load('OCV_SOC_data.mat') % in phd_battery_systems\Lessons\Matlab
% Use the data for the functions (if not already defined)
OCVf = @(z) interp1(SOC_grd,OCVi,z);
SOCf = @(v) interp1(OCVi,SOC_grd,v);

Q = 1.0865*3600; % Capacity (1.0865 Ah) from another exercise

figure
yyaxis left % Create two different scaled y-axis
plot(data.Time ,data.Voltage)
ylabel('Voltage [V]')
yyaxis right % Create the other axis
plot(data.Time ,data.Current)
hold on
plot(data.Time ,data.StepIndex)
ylabel('Current [A]')
title('A123\_dynamic\_data\_sample')

% I take the second part of dynamic data (as we decided)
step_idx = data.StepIndex;
dynamic_data = find(step_idx == 15 | step_idx == 16);
t = data.Time(dynamic_data);
I = data.Current(dynamic_data);
V = data.Voltage(dynamic_data);


%%

% Example for Amina model
[vhat_amina, error_validation_Amina] = battery_model(t, I, V, SOCf, OCVf, Q, 'Linear\_reg');

% Example for Ipek model
[vhat_ipek, error_validation_Ipek] = battery_model(t, I, V, SOCf, OCVf, Q, 'Non\_linear');


function [vhat, error_validation] = battery_model(t, I, V, SOCf, OCVf, Q, model_name)
    
    switch model_name
        case 'Linear\_reg'
            % Amina parameters
            Ri = 0.2;
            R = 0.08;
            C = 10.2 * 1000;
        case 'Non\_linear'
            % Ipek parameters
            Ri = 0.1598;
            R = 0.066;
            C = 1517;
        otherwise
            error('Unknown model name. Please choose either "Linear_reg" or "Non_linear".');
    end

    dt = diff(t);
    n = numel(I);
    
    vc = zeros(n, 1);           
    z = SOCf(V(1));             
    vhat = zeros(1, n);

    for k = 1:n-1
        z(k+1) = z(k) + dt(k) * I(k) / Q;
        vhat(k) = vc(k) + OCVf(z(k)) + Ri * I(k);
        vc(k+1) = vc(k) * (1 - dt(k) / (R * C)) + dt(k) / C * I(k);
    end
    vhat(n) = vc(n) + OCVf(z(n)) + Ri * I(n);

    error_validation = norm(V - vhat);

    % Plotting the results
    figure;
    plot(t, V, 'DisplayName', 'Measurement');
    hold on;
    plot(t, vhat, 'DisplayName', [model_name ' Model']);
    xlabel('Time [s]');
    ylabel('Voltage [V]');
    legend('show');
    title([model_name ' Model vs Measured Voltage']);
    grid on;

    fprintf('Error for the %s model: %.4f\n', model_name, error_validation);
end
