clc
close all
clear all

% This part is included in the Exercise_4_2.mat

data = load("A123_dynamic_data_sample_2.mat");

% Use determined OCV-SOC data from previous exerice
load('OCV_SOC_data.mat') 

% Use the data for the functions (if not already defined)
OCVf = @(z) interp1(SOC_grd,OCVi,z,"linear","extrap");
SOCf = @(v) interp1(OCVi,SOC_grd,v,"linear","extrap");

Q_dis = 1.0865*3600; % Capacity (1.0865 Ah) from another exercise

Ri = 0.2; % Estimated from lesson 3
R = 0.08;
%% a)
SOC = 1:length(OCVi);

figure(1);clf
plot(SOC,OCVi,'LineWidth',2)
xlabel('SOC [%]')
ylabel('Voltage [V]')
grid on
box off

%% Devide the data into the correct intervals

% Plot the voltage and current as function of time
figure(2);clf
yyaxis left % Create two different scaled y-axis
plot(data.Time ,data.Voltage)
ylabel('Voltage [V]')
yyaxis right % Create the other axis
plot(data.Time ,data.Current)
hold on
plot(data.Time ,data.StepIndex)
ylabel('Current [A]')
title('A123\_dynamic\_data\_sample')

% Pick the interval of the second charging curve
step_idx = data.StepIndex;
dynamic_data = find(step_idx >= 4 & step_idx <= 9);
t = data.Time(dynamic_data);
I = data.Current(dynamic_data);
V = data.Voltage(dynamic_data);

% Plot the picked interval
figure(3);clf
yyaxis left % Create two different scaled y-axis
plot(t ,V)
ylabel('Voltage [V]')
yyaxis right % Create the other axis
plot(t ,I)
ylabel('Current [A]')
title('A123\_dynamic\_data\_sample (First cycle)')

%% Voltage based method for SOC estimation

Qint = cumtrapz(t, I);
DOD = max(Qint)-Qint;
True_SOC = 1-DOD/(Q_dis);

SOC_V = SOCf(V);
SOC_Thevenin= SOCf(V - I*(Ri+R)); 

% 1) SOC-estimate and true SOC as a function of time
figure(4);clf
plot(t/3600, True_SOC, LineWidth=2)
hold on
plot(t/3600 ,SOC_V)
legend('True SOC', 'SOCf(V)')
ylabel('SOC')
xlabel('Time [h]')
title('Voltage based method for SOC estimation Vs. True SOC')
figure(5);clf
plot(t/3600, True_SOC, LineWidth=2)
hold on
plot(t/3600 , SOC_Thevenin)
ylabel('SOC')
xlabel('Time [h]')
legend('True SOC', 'SOCf(V - I*(Ri+R))')
title('Voltage based method for SOC estimation Vs. True SOC')

% 2) SOC-error = true SOC - estimated SOC vs time
figure(6);clf
plot(t/3600, abs(True_SOC-SOC_V) * 100)
hold on;
plot(t/3600, abs(True_SOC-SOC_Thevenin) * 100)
sgtitle('SOC error vs time')
legend('SOCf(V)', 'SOCf(V - I*(Ri+R))');

% 3) Compute the RMSE
rmse_socV = mse_soc(True_SOC, SOC_V, t);
rmse_soc_thevenin = mse_soc(True_SOC, SOC_Thevenin, t);


case_names = {'SOC(V)'; 'SOC(V-I*(Ri+R))'};
rmse_values = [rmse_socV; rmse_soc_thevenin];
rmse_table = table(case_names, rmse_values, 'VariableNames', {'Case', 'RMSE'});
disp(rmse_table);

function mse_soc = mse_soc(soc_true, soc_pred, time)
% Calculate the squared error at each time point
squared_error = (soc_true - soc_pred).^2;

% Integrate the squared error over the interval using trapz
integrated_squared_error = trapz(time, squared_error);

% Normalize by the total interval length for MSE approximation
interval_length = time(end) - time(1);
mse_soc = integrated_squared_error / interval_length *100;

end

