clc; clear; close all;

filename = 'A123_dynamic_data_sample_2.mat';
data = load(filename);
load('OCV_SOC_data.mat')
OCVf = @(z) interp1(SOC_grd,OCVi,z,"linear","extrap");
SOCf = @(v) interp1(OCVi,SOC_grd,v,"linear","extrap");
Q_dis = 1.0865*3600; % Capacity (1.0865 Ah) from another exercise

figure
yyaxis left % Create two different scaled y-axis
plot(data.Time ,data.Voltage)
ylabel('Voltage [V]')
yyaxis right % Create the other axis
plot(data.Time ,data.Current)
hold on
plot(data.Time ,data.StepIndex)
ylabel('Current [A]')
title('A123\_dynamic\_data\_sample')

StepIndex = data.StepIndex;
dynamic_data = find(StepIndex >= 4 & StepIndex <= 9);
Time = data.Time(dynamic_data);
Current = data.Current(dynamic_data);
Voltage = data.Voltage(dynamic_data);

figure
yyaxis left % Create two different scaled y-axis
plot(Time ,Voltage)
ylabel('Voltage [V]')
yyaxis right % Create the other axis
plot(Time ,Current)
ylabel('Current [A]')
title('A123\_dynamic\_data\_sample (Fist cycle)')

% Coulumb Counting
Qint = cumtrapz(Time, Current);
DOD = max(Qint)-Qint;
SOCtrue = 1-DOD/(Q_dis);

% Estimated from lesson 3
Ri = 0.2;
R = 0.08;

%% a)
dt = diff(Time); 
n = numel(Time);
z = zeros(size(Time));
z(1) = SOCf(Voltage(1));
Vb_hat = zeros(size(Time));
for k = 1:n-1
    z(k+1) = min(1,z(k) + dt(k) * Current(k)/Q_dis);
    Vb_hat(k) = OCVf(z(k)) + Current(k)*Ri;
end
Vb_hat(n) = OCVf(z(n)) + Current(n)*Ri;

socerror_z = SOCtrue-z;
mean_soc_error_z = trapz(Time(1:end), abs(socerror_z(1:end).^2)*100)/(Time(end)-Time(1))

%% b)
bias = 0.01; 
current_bias = Current + bias;
Qint = cumtrapz(Time, current_bias);
DOD = max(Qint)-Qint;
SOCbias = 1-DOD/(Q_dis);
socerror_bias = SOCtrue-SOCbias;
mean_soc_error_bias = trapz(Time(1:end), abs(socerror_bias(1:end).^2)*100)/(Time(end)-Time(1))

%% C)
Q_large = Q_dis*1.01;
SOClarge = 1-DOD/(Q_large);
socerror_large = SOCtrue-SOClarge;
mean_soc_error_large = trapz(Time(1:end), abs(socerror_large(1:end).^2)*100)/(Time(end)-Time(1))

figure;
plot(Time, SOCtrue * 100)
hold on;
plot(Time, z * 100)
hold on;
plot(Time, SOCbias * 100)
hold on;
plot(Time, SOClarge * 100)
title('True and estimated SOC vs time')
xlabel('Time [s]');
ylabel('State of Charge (SOC) [%]');
legend('SoC true', 'SOCf(Voltage(1))', '0.01 current bias', '1% larger Q');
% Add an inset plot for the ending part
axes('Position', [0.3 0.2 0.25 0.25]); % Adjust inset position and size
box on; % Add a border to the inset
plot(Time, SOCtrue * 100);
hold on;
plot(Time, z * 100);
hold on;
plot(Time, SOCbias * 100);
hold on;
plot(Time, SOClarge * 100);
xlim([Time(end-120) Time(end)]); % Zoom into the last 100 data points
ylim([min(SOCtrue(end-100:end) * 100) - 1, max(SOCtrue(end-100:end) * 100) + 1]); % Adjust y-limits
title('Zoomed-in View at the end');
xlabel('Time [s]');
ylabel('SOC [%]');

figure;
plot(Time, abs(SOCtrue-z) * 100)
hold on;
plot(Time, abs(SOCtrue-SOCbias) * 100)
hold on;
plot(Time, abs(SOCtrue-SOClarge) * 100)
title('SOC error vs time')
xlabel('Time [s]'); 
ylabel('SOC Error [%]'); 
legend('SOCf(Voltage(1))', '0.01 current bias', '1% larger Q');