clear;
clc;
addpath('EKF')

% --- Parameter
SOC_init = 1;       %assume true SOC
x0_init_SOC = 0.5;  %initial guess EKF 
Qc_As = 1.0865*3600;
Ri = 0.28;

% --- Code
data = load('A123_dynamic_data_sample_2.mat');
load('OCV_SOC_data.mat') 

OCVf = @(z) interp1(SOC_grd, OCVi,z,"linear","extrap");
SOCf = @(v) interp1(OCVi,SOC_grd,v,"linear","extrap");

startidx = 1022; % Choose start for a known SOC
endidx = length(data.Time); 
Time = data.Time(startidx:endidx);
Voltage = data.Voltage(startidx:endidx);
Current = data.Current(startidx:endidx);
StepIndex = data.StepIndex(startidx:endidx);


idx = startidx:endidx;
figure(1001);clf;
subplot(3,1,1); hold on; grid on;
plot(data.Time, data.Current,'.b')
plot(Time,Current,'or')
subplot(3,1,2); hold on; grid on;
plot(data.Time,data.Voltage,'.b')
plot(Time,Voltage,'or')
subplot(3,1,3); hold on; grid on;
plot(data.Time,data.StepIndex,'.b')
plot(Time,StepIndex,'or')


Qc_init_As = SOC_init*Qc_As;

ti = Time;
I = Current;
Vt = Voltage;
dt = diff(ti');
dt(end+1) = dt(end);
EKFdata.u = [I; 
             dt'];
EKFdata.y = Vt;

x0 = [x0_init_SOC]; %#ok<NBRAK2>
P0 = [(SOC_grd(end)-SOC_grd(1))*0.06].^2; %#ok<NBRAK1>
mod = struct;
mod.nx = 1; % Number of states
mod.nw = 1; % Dimension of process noise w
mod.f = @(x,u,w)  [x(1)+(u(2)/Qc_As)*(u(1)+w(1))]; %#ok<NBRAK1>
mod.fx = @(x,u,w) [1]; %#ok<NBRAK1>             %Jacobian of df/dx
mod.fw = @(x,u,w) [(u(2)/Qc_As)];%#ok<NBRAK1>   %Jacoian of df/dw

mod.h = @(x,u) (OCVf(x(1)) + Ri*u(1));

dOCV_dSOC = gradient(OCVf(SOC_grd),SOC_grd);
dOCV_dz = @(z) interp1(SOC_grd,dOCV_dSOC,z,"linear","extrap"); 
mod.hx = @(x,u) [dOCV_dz(x(1))];


mod.R = 0.2^2;      % variance in voltage sensor noise
mod.Q = (0.03)^2;   % variance in processes noise, current sensor noise -> SOC
[th_x_hat,th_y_hat,P]=EKF(mod, EKFdata, x0, P0);

th_mod = mod;
th_EKF_data = EKFdata;

DoD = cumtrapz(ti,I) + Qc_init_As;
SOC = DoD/(Qc_As);

n = numel(I);
z = zeros(1,n);
Vt_hat  = zeros(1,n);
%init
z(1) = Qc_init_As/Qc_As;
for k = 1:n-1
    dt = ti(k+1)-ti(k);
    z(k+1) = z(k)+dt*I(k)/Qc_As;    
    Vt_hat(k) = OCVf(z(k)) + Ri*I(k);        
end
Vt_hat(n) = OCVf(z(n)) + Ri*I(n);

figure(1003);clf; 
axh=[];
axh(end+1) = subplot(3,1,1);hold on; grid on;
    % plot(data.Time,data.Current,'-b')
    plot(ti,I,'-m')
    title({sprintf('R = %.5f    Q = %.5f    Q/R = %.5f',mod.R,mod.Q, mod.Q/mod.R)});
axh(end+1) = subplot(3,1,2);hold on; grid on;
    % plot(data.Time,data.Voltage,'-b')
    plot(ti,Vt,'-m')
    plot(ti,Vt_hat,'+c')
    plot(ti,th_y_hat,'og')
axh(end+1) = subplot(3,1,3);hold on; grid on;       
    plot(ti,SOC*100,'-m')
    plot(ti,z*100,'+c')
    plot(ti,th_x_hat*100,'og') 
linkaxes(axh,'x'); 


% Covariance matrices
R_v = 1.0;
R_a = (R_v *[0.1 1.0 10]).^2;
Q_v = 1.0;
Q_a = (Q_v *[0.1 1.0 10]).^2;
ds = struct;
for iiR = 1:numel(R_a)
    for iiQ = 1:numel(Q_a)

        mod.R = R_a(iiR);            % Noise variance in measurement (V)
        mod.Q = Q_a(iiQ);            % Noise varinace in model (SOC)
        [x_hat,y_hat,P]=EKF(mod, EKFdata, x0, P0);
 
        figure(2000+iiR*10+iiQ);clf;         
        axh=[];
        axh(end+1) = subplot(3,1,1);hold on; grid on; %#ok<SAGROW>
            % plot(data.Time,data.Current,'-b')
            plot(ti,I,'-m')
            title({sprintf('R = %.5f    Q = %.5f    Q/R = %.5f',mod.R,mod.Q, mod.Q/mod.R)});
        axh(end+1) = subplot(3,1,2);hold on; grid on; %#ok<SAGROW>
            % plot(data.Time,data.Voltage,'-b')
            plot(ti,Vt,'-m')
            % plot(ti,Vt_hat,'+c')
            % plot(ti,th_y_hat,'oc')
            plot(ti,y_hat,'og')
        axh(end+1) = subplot(3,1,3);hold on; grid on; %#ok<SAGROW>
            plot(ti,SOC*100,'-m')
            % plot(ti,z*100,'+c')
            % plot(ti,th_x_hat*100,'oc') 
            plot(ti,x_hat*100,'oc') 
        linkaxes(axh,'x'); 
                
        dstr = ['t__' num2str(iiR) '_' num2str(iiQ)];        
        ds.(dstr).R = mod.R;
        ds.(dstr).Q = mod.Q;
        ds.(dstr).x_hat = x_hat;
        ds.(dstr).y_hat = y_hat;
    end
end


nR = numel(R_a);
nQ = numel(Q_a);

figure(1007);clf;
iiP = 1;
for iiR = 1:nR
    for iiQ = 1:nQ
        dstr = ['t__' num2str(iiR) '_' num2str(iiQ)];
        z_x_hat = ds.(dstr).x_hat(1,:);
        mR = ds.(dstr).R;
        mQ = ds.(dstr).Q;
        subplot(nQ,nR,iiP); hold on; grid on;
            plot(ti,SOC*100,'.b')
            plot(ti,z_x_hat*100,'.m')   
            ylabel('SOC [%]')
            title({sprintf('R = %.2f    Q = %.2f    Q/R = %.4f',mR,mQ, mQ/mR)});
        iiP = iiP + 1;
    end
end

% From slide 29 Lec5_SOC.pdf:
%   Decrease R -> faster convergance
% From Observation:
%   Too high Q -> instability
%   Filter state xhat SOC z is not great at around 1500s and 2500s
%   Switch to follow measurement at 1700s and 2700s -> model looks better
%   after this.
% From internet
%   A high Q/R ratio means the filter trusts the measurements more than the model.
% Conlustion-> hi
