clear all
clc
addpath('EKF')

data = load('A123_dynamic_data_sample_2.mat');
load('OCV_SOC_data.mat') 

perc = 100; % Set to 1 (if soc from 0-1) or 100 (if SOC 0-100 %)

% Define the function OCVf
OCVf = @(z) conditional_OCV(z, SOC_grd*perc, OCVi);
SOCf = @(v) conditional_SOC(v, SOC_grd*perc, OCVi);


figure;
plot(data.Current)
figure;
plot(data.Voltage)

%%
startidx = 1022; % Choose start for a known SOC
endidx = 8509; 
Time = data.Time(startidx:endidx);
Voltage = data.Voltage(startidx:endidx);
Current = data.Current(startidx:endidx);

% See profile for chosen range
figure;
plot(Current)
figure;
plot(Voltage)
%%
close all;
% Define nonlinear model
mod.nx = 1; % Number of states
mod.nw = 1; % Dimension of process noise w
for ii = 1:1:61
    Q = 1.0865*3600*(ii+100-30)/100;    % Capacity 1.0865*3600
    R = 0.28;           % Sum of R + Ri 
    
    % f function
    mod.f = @(x,u,w) ...
        x + u(2)/Q * (perc*u(1)+w); % NOTE perc here
    
    % h function
    mod.h = @(x,u) ...
        OCVf(x) + R*(u(1)); 
    
    % F matrix
    mod.fx = @(x,u,w) 1;        % Jacobian of df/dx 
    
    % G matrix
    mod.fw = @(x,u,w) u(2)/Q;   % Jacobian of df/dw
    
    SOC = 0:0.01:perc;
    
    dOCVdSOC = gradient(OCVf(SOC), SOC); % Numerical derivative 
    dOCVdz = @(z) conditional_OCV(z, SOC, dOCVdSOC);%interp1(SOC,dOCVdSOC,z);
    
    mod.hx = @(x,u) dOCVdz(x); % Jacobian of dh/dx = H
    
    % Initial conditions
    x0 = SOCf(Voltage(1));  % Inital guess of SOC
    P0 = (0.06*perc)^2;                % Initial uncertainty of init SOC
    
    % Covariance matrices
    mod.R = 0.2^2;                  % Noise variance in measurement (V)
    mod.Q = (0.03*perc)^2;            % Noise varinace in model (SOC)
    
    
    % Input data
    i = Current;
    dt = [0, diff(Time)];
    u = [i;dt];
    
    data.u = u;
    data.y = Voltage; 
    
    % True SOC to comapre results with------------------ 
    trueSOC_ = cumtrapz(Time, Current);
    trueSOC = perc*(trueSOC_/Q + 1);
    
    % Perform EKF
    [xhat,yhat,P]=EKF(mod, data, x0, P0);
    norm_vec(ii) = norm(trueSOC - xhat);
end
%% Plot state estimate
close all;
figure(1)
plot(Time, trueSOC)
hold on
plot(Time, xhat,'k')
legend('True state', 'Estimated state', 'Location','NorthEast')
xlabel('Time (s)')
ylabel('SOC (%)')

norm(trueSOC - xhat)

%% Plot model prediction
figure(2)
plot([Time; Time]',[Voltage; yhat]')
legend('Measurement', 'Estimated output', 'Location','NorthEast')

%%
figure(10);
plot(71:1:131,norm_vec)
ylabel('norm(y-y_{hat})')
xlabel('Percent of the original cpacity [%]')
axis([65 135 0 550])
